#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_portal.interfaces import IPortalTemplate
from pyams_skin.layer import IPyAMSLayer
from pyams_workflow.interfaces import IWorkflowPublicationInfo, IWorkflowCommentInfo, IWorkflowInfo, \
    IWorkflowTransitionInfo

# import packages
from pyams_form.form import AJAXAddForm
from pyams_form.schema import CloseButton
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.url import absolute_url
from pyams_workflow.zmi.transition import WorkflowContentTransitionForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import Interface
from zope.lifecycleevent import ObjectModifiedEvent

from pyams_portal import _


#
# Base workflow form
#

class PortalTemplateWorkflowForm(WorkflowContentTransitionForm):
    """Base portal template workflow form"""


#
# Publish forms
#

class IPortalTemplatePublishButtons(Interface):
    """Portal template publish buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Publish"))


@pagelet_config(name='wf-publish.html', context=IPortalTemplate, layer=IPyAMSLayer,
                permission='portal.templates.manage')
class PortalTemplatePublishForm(PortalTemplateWorkflowForm):
    """Portal template publish form"""

    legend = _("Publish template")

    fields = field.Fields(IWorkflowTransitionInfo) + \
             field.Fields(IWorkflowPublicationInfo).select('publication_effective_date',
                                                           'publication_expiration_date') + \
             field.Fields(IWorkflowCommentInfo)
    buttons = button.Buttons(IPortalTemplatePublishButtons)
    ajax_handler = 'wf-publish.json'

    def createAndAdd(self, data):
        pub_info = IWorkflowPublicationInfo(self.context)
        pub_info.publication_effective_date = data.get('publication_effective_date')
        pub_info.publication_expiration_date = data.get('publication_expiration_date')
        info = IWorkflowInfo(self.context)
        info.fire_transition_toward('published', comment=data.get('comment'))
        self.request.registry.notify(ObjectModifiedEvent(self.context))
        return info


@view_config(name='wf-publish.json', context=IPortalTemplate, request_type=IPyAMSLayer,
             permission='portal.templates.manage', renderer='json', xhr=True)
class PortalTemplateAJAXPublishForm(AJAXAddForm, PortalTemplatePublishForm):
    """Portal template publish form, AJAX renderer"""


#
# Retire form
#

class IPortalTemplateRetireButtons(Interface):
    """Portal template retire buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Retire"))


@pagelet_config(name='wf-retire.html', context=IPortalTemplate, layer=IPyAMSLayer,
                permission='portal.templates.manage')
class PortalTemplateRetireForm(PortalTemplateWorkflowForm):
    """Portal template retire form"""

    legend = _("Retire template")

    buttons = button.Buttons(IPortalTemplateRetireButtons)
    ajax_handler = 'wf-retire.json'


@view_config(name='wf-retire.json', context=IPortalTemplate, request_type=IPyAMSLayer,
             permission='portal.templates.manage', renderer='json', xhr=True)
class PortalTemplateAJAXRetireForm(AJAXAddForm, PortalTemplateRetireForm):
    """Portal template retire form, AJAX renderer"""


#
# Archive form
#

class IPortalTemplateArchiveButtons(Interface):
    """Portal template archive buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Archive"))


@pagelet_config(name='wf-archive.html', context=IPortalTemplate, layer=IPyAMSLayer,
                permission='portal.templates.manage')
class PortalTemplateArchiveForm(PortalTemplateWorkflowForm):
    """Portal template archive form"""

    legend = _("Archive template")

    buttons = button.Buttons(IPortalTemplateArchiveButtons)
    ajax_handler = 'wf-archive.json'


@view_config(name='wf-archive.json', context=IPortalTemplate, request_type=IPyAMSLayer,
             permission='portal.templates.manage', renderer='json', xhr=True)
class PortalTemplateAJAXArchiveForm(AJAXAddForm, PortalTemplateArchiveForm):
    """Portal template archive form, AJAX renderer"""


#
# Clone forms
#

class IPortalTemplateCloneButtons(Interface):
    """Portal template clone buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Create new version"))


@pagelet_config(name='wf-clone.html', context=IPortalTemplate, layer=IPyAMSLayer,
                permission='portal.templates.manage')
class PortalTemplateCloneForm(PortalTemplateWorkflowForm):
    """Portal template clone form"""

    legend = _("Create new version")

    buttons = button.Buttons(IPortalTemplateCloneButtons)
    ajax_handler = 'wf-clone.json'

    def createAndAdd(self, data):
        info = IWorkflowInfo(self.context)
        return info.fire_transition_toward('draft', comment=data.get('comment'))


@view_config(name='wf-clone.json', context=IPortalTemplate, request_type=IPyAMSLayer,
             permission='portal.templates.manage', renderer='json', xhr=True)
class PortalTemplateAJAXCloneForm(AJAXAddForm, PortalTemplateCloneForm):
    """Portal template clone form, AJAX renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'redirect',
                'location': absolute_url(changes, self.request, 'admin.html#properties.html')}
